/**
 * 
 */
package ibspan.ecap.merchant.shopdecisions;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DeliveryLog;
import ibspan.ecap.merchant.shopdecisions.forecasting.ForecastingModule;
import ibspan.ecap.merchant.shopdecisions.negotiationpreparation.NegotiationPreparingModule;

import java.util.Hashtable;

import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

/**
 * A handler behaviour reacting to problems with finding a supplier of a specified product.
 * @author Michal
 *
 */
public class OrderProblemHandler extends OneShotBehaviour {

	private Hashtable<String, Integer> orderProblemCounters;
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	public OrderProblemHandler(Agent myAgent,
			Hashtable<String, Integer> orderProblemCounters) {
		
	}

	/* (non-Javadoc)
	 * @see jade.core.behaviours.Behaviour#action()
	 */
	@Override
	public void action() {
		ACLMessage message = (ACLMessage) getDataStore().get(RecordDeliveriesHandler.MESSAGE_KEY);
		
		if(message == null)
		{
			myLogger.warning("OrderProblemHandler didn't receive a message"); 
			return;
		}
		OrderProblemLog log;
		try
		{
			log = retrieveLog(message);
		} catch (UnreadableException e)
		{
			myLogger.warning("OrderProblemHandler couldn't retrieve content from the message"); 
			return;
		} catch (ClassCastException e)
		{
			myLogger.warning("OrderProblemHandler received wrong content in the message"); 
			return;
		}
		
		String productId = log.getProductId();
		
		Integer counter = orderProblemCounters.get(productId);
		int currentCounterValue = 1;
		if(counter != null && counter.intValue() != 0)
		{
			currentCounterValue = counter.intValue() + 1;
		}
		switch (currentCounterValue) {
		case 1:
			//ignore
			break;
		case 2:
			reactToProblems(productId);
			break;			
		default:
			stopPredicting(productId);
			break;
		}
		
		orderProblemCounters.put(productId, currentCounterValue);

	}

	private void stopPredicting(String productId) {
		myLogger.fine("Withdrawing : "+productId+" from forecasting due to ordering problems.");
		ForecastingModule.getInstance().stopForecasting(productId);
	}

	private void reactToProblems(String productId) {
		myLogger.fine("Preparing new negotiation template for product: "+productId+" due to ordering problems.");
		NegotiationPreparingModule.getInstance().prepareNegotiationDescription(productId);
	}

	private OrderProblemLog retrieveLog(ACLMessage message) throws UnreadableException {
		return (OrderProblemLog) message.getContentObject();
	}

}
