package ibspan.ecap.merchant.shopdecisions;

import java.util.LinkedList;
import java.util.List;

/**
 * A class managing a list of products sold by the shop. Current implementation stores the products in memory.
 * 
 * @author Michal Drozdowicz
 *
 */
public class ProductProvider
{
	private List<String> productsToProcess = new LinkedList<String>();
	private List<String> productsProcessed = new LinkedList<String>();
	
	/**
	 * Returns the id of the next product to process.
	 *
	 * @return the id of the next product to process.
	 */
	public String getNextProductId()
	{
		if (productsToProcess.size() == 0)
		{
			if(productsProcessed.size() == 0)
			{
				return null;	//nothing to process
			}
			else
			{
				productsToProcess = new LinkedList(productsProcessed);
				productsProcessed.clear();
			}
		}
		
		// TODO: Synchronisation needed?
		String productId = productsToProcess.get(0);
		productsToProcess.remove(0);
		productsProcessed.add(productId);
		return productId;
	}
	
	/**
	 * Determines whether a whole cycle of products has passed.
	 *
	 * @return true if all products have been processed during this cycle, false otherwise
	 */
	public boolean roundFinished()
	{
		return productsToProcess.size() == 0;
	}
	
	/**
	 * Adds the product to the list.
	 *
	 * @param productId the id of the product to be handled
	 */
	public void addProduct(String productId)
	{
		if(!productsProcessed.contains(productId) && !productsToProcess.contains(productId))
		{
			productsToProcess.add(productId);			
		}
	}
	
	/**
	 * Removes the product from the list. 
	 *
	 * @param productId the id of the product to be withdrawn
	 */
	public void removeProduct(String productId)
	{
		productsToProcess.remove(productId);
		productsProcessed.remove(productId);		
	}
}
