package ibspan.ecap.merchant.shopdecisions;

import java.util.logging.Level;

import ibspan.ecap.merchant.shopdecisions.forecasting.ForecastingModule;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

/**
 * A behaviour responding to a product run out event. It does so by immediately scheduling
 * the preparation of a new sales forecast.   
 * 
 * @author Micha Drozdowicz
 *
 */
public class ProductRunOutHandler extends OneShotBehaviour {

	public static final String MESSAGE_KEY = "MessageKey";
	
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName()); 
			
	public ProductRunOutHandler(Agent myAgent)
	{
		super(myAgent);
	}
	
	@Override
	public void action() {
		ACLMessage message = (ACLMessage) getDataStore().get(ProductRunOutHandler.MESSAGE_KEY);
		
		if(message == null)
		{
			myLogger.warning("ProductRunOutHandler didn't receive a message"); 
			return;
		}
		
		String productId = null;		
		try {
			productId = retrieveProductId(message);
		} catch (UnreadableException e) {
			if(myLogger.isLoggable(Level.WARNING))
				myLogger.warning("ProductRunOutHandler couldn't read the message. Exception: "+e.toString());
			return;
		}
		
		ForecastingModule module = ForecastingModule.getInstance();
		if(myLogger.isLoggable(Level.INFO))
			myLogger.info("Submitting a forecast request for product: "+productId);
		module.prepareForecast(productId);
	}

	protected String retrieveProductId(ACLMessage message) throws UnreadableException {
		return (String) message.getContent();
	}

}
