package ibspan.ecap.merchant.shopdecisions;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAccessException;
import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAdministrationModule;
import ibspan.ecap.merchant.shopdecisions.negotiationpreparation.NegotiationTemplateUpdater;
import ibspan.ecap.shared.ontology.negotiation.DemandData;
import ibspan.ecap.shared.ontology.negotiation.NegotiationLog;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

public class RecordDemandDataHandler extends OneShotBehaviour
{
	public static final String MESSAGE_KEY = "MessageKey";
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	public RecordDemandDataHandler(Agent myAgent)
	{
		super(myAgent);
	}

	@Override
	public void action()
	{
		ACLMessage message = (ACLMessage) getDataStore().get(RecordDemandDataHandler.MESSAGE_KEY);
		
		if(message == null)
		{
			myLogger.warning("RecordDemandDataHandler didn't receive a message"); 
			return;
		}
		DemandData log;
		try
		{
			log = retrieveLog(message);
		} catch (UnreadableException e)
		{
			myLogger.warning("RecordDemandDataHandler couldn't retrieve content from the message"); 
			return;
		} catch (ClassCastException e)
		{
			myLogger.warning("RecordDemandDataHandler received wrong content in the message"); 
			return;
		}
		
		try
		{
			DataAdministrationModule.getInstance().recordDemandData(log);
		} catch (DataAccessException e)
		{
			myLogger.warning("RecordNegotiationHandler failed to record demand data."); 
			return;
		}
		
		//Test scenario only. Request the preparation of negotation params.
		//myAgent.addBehaviour(new NegotiationTemplateUpdater(myAgent, 0, false));
		
	}

	private DemandData retrieveLog(ACLMessage message) throws UnreadableException
	{
		DemandData log = (DemandData) message.getContentObject();
		return log;
	}

}
