package ibspan.ecap.merchant.shopdecisions;

import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAccessException;
import ibspan.ecap.merchant.shopdecisions.dataadministration.DataAdministrationModule;
import ibspan.ecap.merchant.shopdecisions.dataadministration.InvalidOperationException;
import ibspan.ecap.merchant.shopdecisions.dataadministration.TransactionLog;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

import java.util.logging.Level;

/**
 * A handler behaviour responsible for recording information about a finished transaction.sS
 * 
 * @author Micha Drozdowicz
 *
 */
public class RecordTransactionHandler extends OneShotBehaviour
{
	
	private static final long serialVersionUID = -6876206660349020872L;
	public static final String MESSAGE_KEY = "MessageKey";
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	public RecordTransactionHandler(Agent myAgent)
	{
		super(myAgent);
	}

	@Override
	public void action()
	{
//		ACLMessage message = (ACLMessage) getDataStore().get(RecordNegotiationHandler.MESSAGE_KEY);
//		
//		if(message == null)
//		{
//			myLogger.warning("RecordTransactionHandler didn't receive a message"); 
//			return;
//		}
//		TransactionLog log;
//		try
//		{
//			log = retrieveLog(message);
//		} catch (UnreadableException e)
//		{
//			myLogger.warning("RecordTransactionHandler couldn't retrieve content from the message"); 
//			return;
//		} catch (ClassCastException e)
//		{
//			myLogger.warning("RecordTransactionHandler received wrong content in the message"); 
//			return;
//		}
//		
//		try
//		{
//			DataAdministrationModule.getInstance().recordFinishedTransactionDetails(log);
//		} catch (DataAccessException e)
//		{
//			myLogger.log(Level.WARNING, "RecordNegotiationHandler failed to record transaction data.", e);
//			return;
//		} catch (InvalidOperationException e)
//		{
//			myLogger.log(Level.WARNING, "RecordNegotiationHandler failed to record transaction data.", e);
//			return;
//		}
		
		ACLMessage message = (ACLMessage) getDataStore().get(RecordNegotiationHandler.MESSAGE_KEY);
		
		if(message == null)
		{
			myLogger.warning("RecordTransactionHandler didn't receive a message"); 
			return;
		}
		TransactionLog[] logs;
		try
		{
			logs = retrieveLog(message);
		} catch (UnreadableException e)
		{
			myLogger.warning("RecordTransactionHandler couldn't retrieve content from the message"); 
			return;
		} catch (ClassCastException e)
		{
			myLogger.warning("RecordTransactionHandler received wrong content in the message"); 
			return;
		}
		
		myLogger.info("Received transaction logs. Size:"+logs.length);
		for (TransactionLog log : logs) {

			try {
				DataAdministrationModule.getInstance()
						.recordFinishedTransactionDetails(log);
			} catch (DataAccessException e) {
				myLogger
						.log(
								Level.WARNING,
								"RecordNegotiationHandler failed to record transaction data.",
								e);
				return;
			} catch (InvalidOperationException e) {
				myLogger
						.log(
								Level.WARNING,
								"RecordNegotiationHandler failed to record transaction data.",
								e);
				return;
			}
		}
	}
	
	private TransactionLog[] retrieveLog(ACLMessage message) throws UnreadableException
	{
		TransactionLog[] log = (TransactionLog[]) message.getContentObject();
		return log;
	}

}
