package ibspan.ecap.merchant.shopdecisions;

import java.util.logging.Level;

import ibspan.ecap.merchant.shopdecisions.forecasting.ForecastingModule;
import jade.core.Agent;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

/**
 * A behaviour handling the request for starting the sales forecast of a product. It does so by
 * registering the product for periodical forecasting and scheduling the first forecast.
 *
 * @author Micha Drozdowicz
 *
 */
public class StartForecastingHandler extends OneShotBehaviour {

	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName()); 

	public static final String MESSAGE_KEY = "MessageKey";
	
	public StartForecastingHandler(Agent myAgent)
	{
		super(myAgent);
	}
	
	@Override
	public void action() {
		ACLMessage message = (ACLMessage) getDataStore().get(StartForecastingHandler.MESSAGE_KEY);
				
		if(message == null)
		{
			myLogger.warning("StartForecasting didn't receive a message"); 
			return;
		}
		
		String productId = null;		
		try {
			productId = retrieveProductId(message);
		} catch (UnreadableException e) {
			if(myLogger.isLoggable(Level.WARNING))
				myLogger.warning("StartForecasting couldn't read the message. Exception: "+e.toString());
			return;
		}
		
		ForecastingModule module = ForecastingModule.getInstance();
		if(myLogger.isLoggable(Level.INFO))
			myLogger.info("Starting forecasting of product: "+productId);
		module.startForecasting(productId);
	}

	protected String retrieveProductId(ACLMessage message) throws UnreadableException {
		return (String) message.getContentObject();
	}
}
