package ibspan.ecap.merchant.shopdecisions.dataaccess;

/**
 * A class representing demand for a product. It is roughly equivalent to an entity of the Demand fact table.
 * 
 * @author  Micha Drozdowicz
 */
public class DemandFact
{
	private int dateKey;
	private int productKey;
	private int cicQueryCount;
	private int shopsSellingCount;
	private int registeredClients;
	
	

	/**
	 * Instantiates a new demand fact.
	 */
	public DemandFact(){};
	
	/**
	 * Instantiates a new demand fact.
	 * 
	 * @param dateKey the key to the date dimension entry that describes the date when the demand has been checked.
	 * @param productKey the key to the product dimension entry that describes the product for which the demand was checked.
	 * @param cicQueryCount the cic query count
	 * @param shopsSellingCount the shops selling count
	 */
	public DemandFact(int dateKey, int productKey, int cicQueryCount,
			int shopsSellingCount)
	{
		this.dateKey = dateKey;
		this.productKey = productKey;
		this.cicQueryCount = cicQueryCount;
		this.shopsSellingCount = shopsSellingCount;
	}

	/**
	 * Gets the key to the date dimension entry that describes the date when the demand has been checked.
	 * 
	 * @return the key to the date dimension entry that describes the date when the demand has been checked.
	 */
	public int getDateKey()
	{
		return dateKey;
	}

	/**
	 * Sets the key to the date dimension entry that describes the date when the demand has been checked.
	 * 
	 * @param dateKey the key to the date dimension entry that describes the date when the demand has been checked.
	 */
	public void setDateKey(int dateKey)
	{
		this.dateKey = dateKey;
	}

	/**
	 * Gets the key to the product dimension entry that describes the product for which the demand was checked.
	 * 
	 * @return the key to the product dimension entry that describes the product for which the demand was checked.
	 */
	public int getProductKey()
	{
		return productKey;
	}

	/**
	 * Sets the key to the product dimension entry that describes the product for which the demand was checked.
	 * 
	 * @param productKey the key to the product dimension entry that describes the product for which the demand was checked.
	 */
	public void setProductKey(int productKey)
	{
		this.productKey = productKey;
	}

	/**
	 * Gets the number of queries about the product sent to the cic, the day the demand was checked.
	 * 
	 * @return the number of queries about the product sent to the cic, the day the demand was checked.
	 */
	public int getCicQueryCount()
	{
		return cicQueryCount;
	}

	/**
	 * Sets the number of queries about the product sent to the cic, the day the demand was checked.
	 * 
	 * @param cicQueryCount the number of queries about the product sent to the cic, the day the demand was checked.
	 */
	public void setCicQueryCount(int cicQueryCount)
	{
		this.cicQueryCount = cicQueryCount;
	}

	/**
	 * Gets the number of shops selling the product on the day the demand was checked
	 * 
	 * @return the number of shops selling the product on the day the demand was checked
	 */
	public int getShopsSellingCount()
	{
		return shopsSellingCount;
	}

	/**
	 * Sets the number of shops selling the product on the day the demand was checked
	 * 
	 * @param shopsSellingCount the number of shops selling the product on the day the demand was checked
	 */
	public void setShopsSellingCount(int shopsSellingCount)
	{
		this.shopsSellingCount = shopsSellingCount;
	}	
	
	/**
	 * Gets the total number of clients that registered with the GA wanting to buy the product.
	 * 
	 * @return the total number of clients that registered with the GA wanting to buy the product.
	 */
	public int getRegisteredClients()
	{
		return registeredClients;
	}

	/**
	 * Sets the total number of clients that registered with the GA wanting to buy the product.
	 * 
	 * @param registeredClients the total number of clients that registered with the GA wanting to buy the product.
	 */
	public void setRegisteredClients(int registeredClients)
	{
		this.registeredClients = registeredClients;
	}
}
