package ibspan.ecap.merchant.shopdecisions.dataaccess.tests;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.DateEntity;
import ibspan.ecap.merchant.shopdecisions.dataaccess.ProductEntity;
import ibspan.ecap.merchant.shopdecisions.dataaccess.SalesFact;
import ibspan.ecap.merchant.shopdecisions.dataaccess.StrategyEntity;
import ibspan.ecap.merchant.shopdecisions.dataaccess.TemplateEntity;
import ibspan.ecap.merchant.shopdecisions.dataaccess.TimeEntity;
import ibspan.ecap.merchant.shopdecisions.dataaccess.WholesalerEntity;
import junit.framework.TestCase;

public class DataAccessModuleTest extends TestCase
{

	public void testGetDateKey()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		Calendar calendar = new GregorianCalendar(2006, 11, 25);		
		try
		{
			id = dms.getDateKey(calendar.getTime());
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{			
			Date date = calendar.getTime();
			int newId = -10;
			try
			{
				newId = dms.insertDate(new DateEntity(
						date, 
						getDayOfWeekName(calendar.get(Calendar.DAY_OF_WEEK)),
						calendar.get(Calendar.DAY_OF_WEEK+1),
						getDayInEpoch(date),
						getWeekInEpoch(date),
						getMonthInEpoch(calendar),
						calendar.get(Calendar.DAY_OF_YEAR),
						calendar.get(Calendar.WEEK_OF_YEAR),
						calendar.get(Calendar.MONTH+1),
						calendar.get(Calendar.MONTH+1)/4 + 1,
						calendar.get(Calendar.YEAR),
						calendar.get(Calendar.DAY_OF_MONTH),
						getWeekdayIndicator(calendar)));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert date. Exception: "+e.toString());
			}					
			
			try
			{
				System.out.println(date.toString());
				id = dms.getDateKey(date);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}
	
	private String getWeekdayIndicator(Calendar calendar)
	{		
		int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);		
		if( dayOfWeek == Calendar.SATURDAY || dayOfWeek == Calendar.SUNDAY)
			return "Weekday";
		else
			return "Weekend";
	}

	private String getDayOfWeekName(int dayInWeek)
	{	
		switch (dayInWeek)
		{
		case Calendar.MONDAY:
			return "Monday";
		case Calendar.TUESDAY:
			return "Tuesday";
		case Calendar.WEDNESDAY:
			return "Wednesday";
		case Calendar.THURSDAY:
			return "Thursday";
		case Calendar.FRIDAY:
			return "Friday";
		case Calendar.SATURDAY:
			return "Saturday";
		case Calendar.SUNDAY:
			return "Sunday";
		}
				
		return null;		
	}
	
	private int getDayInEpoch(Date date)
	{
		int millisInDay = 1000 * 60 * 60 * 24;
		
		return (int) date.getTime() / millisInDay;
	}
	
	private int getMonthInEpoch(Calendar cal)
	{
		int years = cal.get(Calendar.YEAR) - 1970;
		
		return years * 12 + cal.get(Calendar.MONTH);
	}
	
	private int getWeekInEpoch(Date date)
	{
		return getDayInEpoch(date) / 7 + 1;
	}

	
	public void testTimeDimension()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		Calendar calendar = new GregorianCalendar(2006, 11, 25, 15, 32, 30);		
		try
		{
			id = dms.getTimeKey(calendar.getTime());
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{			
			Date date = calendar.getTime();
			int newId = -10;
			try
			{
				newId = dms.insertTime(new TimeEntity(
						date, 
						calendar.get(Calendar.HOUR_OF_DAY),
						getAmPmIndicator(calendar)));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert time. Exception: "+e.toString());
			}					
			
			try
			{
				System.out.println(date.toString());
				id = dms.getTimeKey(date);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}

	private String getAmPmIndicator(Calendar calendar)
	{
		if(calendar.get(Calendar.AM_PM) == Calendar.AM)
			return "AM";
		else
			return "PM";
	}
	
	public void testProductDimension()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		String productId = "Blender ZX-2010-UDPA";		
		try
		{
			id = dms.getProductKey(productId);
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{				
			int newId = -10;
			try
			{
				newId = dms.insertProduct(new ProductEntity(
						productId));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert. Exception: "+e.toString());
			}					
			
			try
			{				
				id = dms.getProductKey(productId);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}
	
	public void testWholesalerDimension()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		String globalId = "J. Burczymucha Ltd.";		
		try
		{
			id = dms.getWholesalerKey(globalId);
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{				
			int newId = -10;
			try
			{
				newId = dms.insertWholesaler(new WholesalerEntity(
						globalId));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert. Exception: "+e.toString());
			}					
			
			try
			{				
				id = dms.getWholesalerKey(globalId);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}
	
	public void testStrategyDimension()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		String globalId = "Dutch";
		String description = "Dutch auction.";
		try
		{
			id = dms.getStrategyKey(globalId);
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{				
			int newId = -10;
			try
			{
				newId = dms.insertStrategy(new StrategyEntity(
						globalId, description));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert. Exception: "+e.toString());
			}					
			
			try
			{				
				id = dms.getStrategyKey(globalId);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}
	
	public void testTemplateDimension()
	{
		int id = -10;	//invalid value
		DataModificationService dms = DataModificationService.getInstance();
		String globalId = "Dutch";
		String description = "Dutch auction.";
		try
		{
			id = dms.getTemplateKey(globalId);
		} catch (SQLException e)
		{			
			e.printStackTrace();
			fail(e.toString());
		}
		if(id == -10)
		{
			fail("Invalid value");
		}
		if(id == 0) //date not found
		{				
			int newId = -10;
			try
			{
				newId = dms.insertTemplate(new TemplateEntity(
						globalId, description));				
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to insert. Exception: "+e.toString());
			}					
			
			try
			{				
				id = dms.getTemplateKey(globalId);
			} catch (SQLException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
				fail("Failed to get date. Exception: "+e.toString());
			}
			
			assertEquals(id, newId);
			System.out.println("Inserted");
			return;
		}
		//found key
		System.out.println("Found key: "+id);
	}
	
	public void testSalesFact()
	{
		DataModificationService dms = DataModificationService.getInstance();
		int negotiationId = 1;
		try
		{
			SalesFact fact = dms.getSalesFact(negotiationId);
			assertNotNull(fact);			
			assertEquals(fact.getNegotiationId(), negotiationId);
		} catch (SQLException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
			fail(e.toString());
		}
		
		
		
			
	}
}
