package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.DateEntity;
import ibspan.ecap.shared.util.DateTool;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * A class responsible for handling and staging the date dimension.
 * 
 * @author Micha Drozdowicz
 */
public class DateProcessor
{
	private DataModificationService dms;
	
	private static Object padlock = new Object();

	/**
	 * Instantiates a new date processor.
	 * 
	 * @param service the {@link DataModificationService} to use to access the database
	 */
	public DateProcessor(DataModificationService service)
	{
		dms = service;
	}
	
	/**
	 * Retrieve date key.
	 * 
	 * @param date the date, the key of which is to be retrieved.
	 * 
	 * @return the the key of the date dimension entity
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveDateKey(Date date) throws SQLException
	{
		synchronized (padlock) {
			Calendar calendar = GregorianCalendar.getInstance();
			calendar.setTime(date);
//			calendar.set(Calendar.HOUR_OF_DAY, 12);
//			calendar.set(Calendar.MINUTE, 0);
//			calendar.set(Calendar.SECOND, 0);
//			calendar.set(Calendar.MILLISECOND, 0);
			int key = dms.getDateKey(date);
			if (key > 0) {
				return key;
			} else {
				key = dms.insertDate(createDateEntity(date));
				return key;
			}
		}
	}
	
	private DateEntity createDateEntity(Date date)
	{
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.setTime(date);

//		calendar.set(Calendar.HOUR_OF_DAY, 12);
//		calendar.set(Calendar.MINUTE, 0);
//		calendar.set(Calendar.SECOND, 0);
//		calendar.set(Calendar.MILLISECOND, 0);
		
		DateEntity de = new DateEntity(
				date, 
				DateTool.getDayOfWeekName(calendar.get(Calendar.DAY_OF_WEEK)),
				DateTool.getDayOfWeek(calendar.get(Calendar.DAY_OF_WEEK)),
				DateTool.getDayInEpoch(date),
				DateTool.getWeekInEpoch(date),
				DateTool.getMonthInEpoch(calendar),
				calendar.get(Calendar.DAY_OF_YEAR),
				calendar.get(Calendar.WEEK_OF_YEAR),
				calendar.get(Calendar.MONTH)+1,
				(calendar.get(Calendar.MONTH)+1)/4 + 1,
				calendar.get(Calendar.YEAR),
				calendar.get(Calendar.DAY_OF_MONTH),
				DateTool.getWeekdayIndicator(calendar));
		return de;
	}

	
	
	

	
}
