package ibspan.ecap.merchant.shopdecisions.dataadministration;

import java.io.Serializable;
import java.util.Date;

/**
 * A class containing information about a delivery to the warehouse. 
 *
 * @author Micha Drozdowicz
 *
 */
public class DeliveryLog implements Serializable
{
	private String deliveredProductId;
	private Date orderTime;
	private Date deliveryTime;
	private int orderedQuantity;
	private float orderedValue;
	private String wholesalerId;
	
	/**
	 * Gets the id of the wholesaler, that carried out the order.
	 *
	 * @return the id of the wholesaler, that carried out the order.
	 */
	public String getWholesalerId()
	{
		return wholesalerId;
	}
	/**
	 * Sets the id of the wholesaler, that carried out the order.
	 *
	 * @param wholesalerId the id of the wholesaler, that carried out the order.
	 */
	public void setWholesalerId(String wholesalerId)
	{
		this.wholesalerId = wholesalerId;
	}
	/**
	 * Gets the id of the delivered product.
	 *
	 * @return the id of the delivered product
	 */
	public String getDeliveredProductId()
	{
		return deliveredProductId;
	}
	/**
	 * Sets the id of the delivered product
	 *
	 * @param deliveredProductId the id of the delivered product to set
	 */
	public void setDeliveredProductId(String deliveredProductId)
	{
		this.deliveredProductId = deliveredProductId;
	}
	/**
	 * Gets the time when the product was ordered.
	 *
	 * @return the the time when the product was ordered
	 */
	public Date getOrderTime()
	{
		return orderTime;
	}
	/**
	 * Sets the time when the product was ordered
	 *
	 * @param orderTime the time when the product was ordered
	 */
	public void setOrderTime(Date orderTime)
	{
		this.orderTime = orderTime;
	}
	/**
	 * Gets the time when the product was delivered.
	 *
	 * @return the time when the product was delivered
	 */
	public Date getDeliveryTime()
	{
		return deliveryTime;
	}
	/**
	 * Sets the time when the product was delivered
	 *
	 * @param deliveryTime the time when the product was delivered
	 */
	public void setDeliveryTime(Date deliveryTime)
	{
		this.deliveryTime = deliveryTime;
	}
	/**
	 * Gets the ordered amount of product.
	 *
	 * @return the ordered amount of product.
	 */
	public int getOrderedQuantity()
	{
		return orderedQuantity;
	}
	/**
	 * Sets the ordered amount of product.
	 *
	 * @param orderedQuantity the ordered amount of product.
	 */
	public void setOrderedQuantity(int orderedQuantity)
	{
		this.orderedQuantity = orderedQuantity;
	}
	/**
	 * Gets the total value of the ordered product.
	 *
	 * @return the total value of the ordered product.
	 */
	public float getOrderedValue()
	{
		return orderedValue;
	}
	/**
	 * Sets the total value of the ordered product.
	 *
	 * @param orderedValue the total value of the ordered product
	 */
	public void setOrderedValue(float orderedValue)
	{
		this.orderedValue = orderedValue;
	}
	
	@Override
	public String toString() {
		return String.format("DeliveryLog: {\n" +
				"\tdeliveredProductId: %s\n" +
				"\torderTime: %s\n" +
				"\tdeliveryTime: %s\n" +
				"\torderedQuantity: %s\n" +
				"\torderedValue: %s\n" +
				"\twholesalerId: %s\n" +
				"}\n", 
				deliveredProductId, orderTime, deliveryTime, orderedQuantity,
				orderedValue, wholesalerId);
	}
}
