package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DemandFact;
import ibspan.ecap.shared.ontology.negotiation.DemandData;


/**
 * A class responsible for extracting the appropriate information from the demand details and creating a supplies fact object.
 * 
 * @author Micha Drozdowicz
 *
 */
public class DemandFactProcessor
{
	
	/**
	 * Creates the demand fact.
	 * 
	 * @param productKey the key to the product dimension entry that describes the product for which the demand was checked.
	 * @param dateKey the key to the date dimension entry that describes the date when the demand has been checked.
	 * @param data the demand data
	 * 
	 * @return the demand fact
	 */
	public DemandFact createDemandFact(int productKey, int dateKey,
			DemandData data)
	{
		DemandFact fact = new DemandFact();
		fact.setProductKey(productKey);
		fact.setDateKey(dateKey);
		fact.setCicQueryCount(data.getCicQueryCount());
		fact.setShopsSellingCount(data.getShopsSellingProductCount());
		fact.setRegisteredClients(data.getRegisteredClients());
		
		return fact;
	}
	/**
	 * Updates the demand fact with new information about the demand gathered from the CIC.
	 *
	 * @param fact the fact data to update
	 */
	public void updateDemandFact(DemandFact fact, DemandData data)
	{
		fact.setCicQueryCount(data.getCicQueryCount());
		fact.setShopsSellingCount(data.getShopsSellingProductCount());		
	}
	
	
	/**
	 * Creates the demand fact filling it with information about the number of buyers registered wanting to buy the product.
	 * 
	 * @param productKey the product key
	 * @param dateKey the date key
	 * @param registeredClients the registered clients
	 * 
	 * @return the demand fact
	 */
	public DemandFact createDemandFact(int productKey, int dateKey, int registeredClients)
	{
		DemandFact fact = new DemandFact();
		fact.setProductKey(productKey);
		fact.setDateKey(dateKey);
		fact.setRegisteredClients(registeredClients);
		return fact;
	}
	
	/**
	 * Updates the demand fact with new information about the number of clients registered to buy the product.
	 *
	 * @param fact the fact data to update
	 * @param registeredClients the number of clients registered to buy the product.
	 */
	public void updateDemandFact(DemandFact fact, int registeredClients)
	{
		fact.setRegisteredClients(fact.getRegisteredClients() + registeredClients);
	}	
}
