package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.TemplateEntity;

import java.sql.SQLException;

/**
 * A class responsible for handling and staging the template dimension.
 * 
 * @author Micha Drozdowicz
 */
public class TemplateProcessor
{
	private DataModificationService dms;
	private static Object padlock = new Object();
	
	/**
	 * Instantiates a new template processor.
	 * 
	 * @param svc the {@link DataModificationService} to use to access the database
	 */
	public TemplateProcessor(DataModificationService svc)
	{
		dms = svc;
	}

	/**
	 * Retrieve template key.
	 * 
	 * @param templateId the id of the template, the key of which is to be retrieved.
	 * 
	 * @return the the key of the template
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveTemplateKey(String templateId) throws SQLException
	{
		synchronized (padlock) {
			int key = dms.getTemplateKey(templateId);
			if (key > 0) {
				return key;
			} else {
				key = dms.insertTemplate(createTemplateEntity(templateId));
				return key;
			}
		}		
	}

	private TemplateEntity createTemplateEntity(String templateId)
	{
		return new TemplateEntity(templateId, "Unknown template");
	
	}

}
