package ibspan.ecap.merchant.shopdecisions.dataadministration;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataModificationService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.TimeEntity;

import java.sql.SQLException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * A class responsible for handling and staging the time dimension.
 * 
 * @author Micha Drozdowicz
 */
public class TimeProcessor
{
	private DataModificationService dms;
	private static Object padlock = new Object();

	/**
	 * Instantiates a new time processor.
	 * 
	 * @param service the {@link DataModificationService} to use to access the database
	 */
	public TimeProcessor(DataModificationService service)
	{
		dms = service;
	}
	
	/**
	 * Retrieve time key.
	 * 
	 * @param date the date, the key of which is to be retrieved.
	 * 
	 * @return the the key of the time
	 * 
	 * @throws SQLException the SQL exception
	 */
	public int retrieveTimeKey(Date date) throws SQLException
	{
		synchronized (padlock) {
			int key = dms.getTimeKey(date);
			if (key > 0) {
				return key;
			} else {
				key = dms.insertTime(createTimeEntity(date));
				return key;
			}
		}		
	}

	private TimeEntity createTimeEntity(Date date)
	{
		Calendar calendar = GregorianCalendar.getInstance();
		calendar.setTime(date);
		
		TimeEntity te = new TimeEntity(
				date, 
				calendar.get(Calendar.HOUR_OF_DAY),
				getAmPmIndicator(calendar));
		return te;
	}
	
	private String getAmPmIndicator(Calendar calendar)
	{
		if(calendar.get(Calendar.AM_PM) == Calendar.AM)
			return "AM";
		else
			return "PM";
	}
}
