package ibspan.ecap.merchant.shopdecisions.forecasting;

import java.util.Date;

import net.sourceforge.openforecast.DataPoint;
import net.sourceforge.openforecast.DataSet;

/**
 * An interface describing a set of methods creating data collections required by the forecast
 * evaluator to prepare the prediction.
 *
 * @author Micha Drozdowicz
 *
 */
public interface DataProvider
{

	/**
	 * Creates a {@link DataSet} containing historical data needed to perform the forecast. 
	 *
	 * @param productId the global id of the product to be forecasted 
	 * @param startDate the date starting from which the sales should be forecasted
	 * @param periodLength the length (in days) of the forecast period
	 * @return a {@link DataSet} containing historical data needed to perform the forecast
	 * @throws Exception when the retrieval of the data was unsuccessful 
	 */
	public DataSet getHistoricalData(String productId, Date startDate,
			int periodLength) throws Exception;

	/**
	 * Creates a {@link DataPoint} containing the known (independent) future variables used by 
	 * the forecast evaluator to predict the sales volume of a product. 
	 *
	 * @param productId the global id of the product to be forecasted 
	 * @param startDate the date starting from which the sales should be forecasted
	 * @param periodLength the length (in days) of the forecast period
	 * @return a {@link DataSet} containing historical data needed to perform the forecast
	 * @return a {@link DataPoint} containing the future values of the independent variables
	 * @throws Exception 
	 */
	public DataPoint getFuturePoint(String productId, Date startDate,
			int periodLength, DataSet historicalData) throws Exception;
}
