package ibspan.ecap.merchant.shopdecisions.forecasting;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataQueryService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.ResultProcessor;
import ibspan.ecap.shared.util.DateTool;

import jade.util.Logger;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.sourceforge.openforecast.DataPoint;
import net.sourceforge.openforecast.DataSet;
import net.sourceforge.openforecast.Observation;

public class DatabaseDataProvider implements DataProvider
{
	private String timeVariableName = "period";
	private Logger myLogger = Logger.getMyLogger(this.getClass().getName());
	
	public DataPoint getFuturePoint(String productId, Date startDate,
			int periodLength, DataSet historicalData) throws Exception
	{
		Date minDay = getMinDay();	//first day from which we have data
		int startDay = DateTool.getDayInEpoch(startDate) - DateTool.getDayInEpoch(minDay);
		myLogger.finer("minDay:"+minDay);
		myLogger.finer("startDay:"+startDay);
		int periodNumber = (startDay - startDay % periodLength)/periodLength;
		myLogger.finer("periodNo:"+periodNumber);
		
		DataPoint point = new Observation(0);
		point.setIndependentValue(timeVariableName, periodNumber);
		
		return point;
	}

	private Date getMinDay() throws SQLException {
		
		String query = "SELECT MIN(ddt_fulldate) as minDate FROM dim_date";
		
		final Date minDay = new Date();
		
		DataQueryService.getInstance().executeQuery(query, new ResultProcessor()
		{
			public void processResult(ResultSet rs) throws SQLException
			{
				if(rs.next())
				{
					Date minDate = rs.getDate("minDate");
					if(minDate!=null)
						minDay.setTime(minDate.getTime());
					else
						minDay.setTime(0);
				}
							
			}
		});
		
		if(minDay.getTime()>0)
			return minDay;
		else
			return null;
	}

	public DataSet getHistoricalData(String productId, Date startDate,
			int periodLength) throws Exception
	{		
		int startDay = DateTool.getDayInEpoch(startDate);
		int shift = startDay % periodLength;
		myLogger.finer("startDay:"+startDay);
		myLogger.finer("shift:"+shift);
		
		Date minDate = getMinDay();
		
		if(minDate == null)
			return new DataSet();
		
		int minDay = DateTool.getDayInEpoch(minDate);
		myLogger.finer("minDay:"+minDay);
		String query = "SELECT (ddt_dayinepoch-"+minDay+"-" + shift + ")/" + periodLength+ "  as periodnumber,"
				+ " COALESCE(SUM(fsa_quantityconfirmed),0) as quantityconfirmed FROM ft_sales"
				+ " RIGHT JOIN dim_product ON fsa_dpd_productid=dpd_id"
				+ " RIGHT JOIN dim_date ON fsa_ddt_negoenddateid=ddt_id"
				+ " WHERE (dpd_globalproductid = '" + productId
				+ "' OR dpd_globalproductid IS NULL)" 
				+ " AND ddt_fulldate < '"	+ DateFormat.getDateInstance().format(startDate) + "'"
				+ " GROUP BY (ddt_dayinepoch-"+minDay+"-" + shift + ")/" + periodLength
				+ " ORDER BY (ddt_dayinepoch-"+minDay+"-" + shift + ")/" + periodLength;
		
		
		final DataSet data = new DataSet();
		data.setPeriodsPerYear(365/periodLength);
		data.setTimeVariable(timeVariableName);

		DataQueryService.getInstance().executeQuery(query,
				new ResultProcessor() {
					public void processResult(ResultSet rs) throws SQLException {
						boolean nonZeroFound = false;
						while (rs.next()) {
							int periodNo = rs.getInt("periodnumber");
							int quantityConfirmed = rs
									.getInt("quantityconfirmed");
							myLogger.finest("periodNo:" + periodNo);
							myLogger.finest("quantityConfirmed:"
									+ quantityConfirmed);
							if (quantityConfirmed > 0 && !nonZeroFound)
								nonZeroFound = true;
							if (nonZeroFound) {
								DataPoint point = new Observation(
										quantityConfirmed);
								point.setIndependentValue(timeVariableName, periodNo);
								myLogger.finest("Adding data periodNo:"
										+ periodNo);
								myLogger.finest("Adding data quantityConfirmed:"
										+ quantityConfirmed);
								data.add(point);
							}
						}
					}
				});

		return data;
	}
	
		
}
