package ibspan.ecap.merchant.shopdecisions.forecasting;

import java.util.Date;

/**
 * Defines a method to estimate a forecast horizon / period for a specified product and forecast
 * starting date.
 *
 * @author Micha Drozdowicz
 *
 */
public interface HorizonEstimator
{
	/**
	 * Returns the forecast period length (in days) for the prediction of the given product
	 * starting at the specified date. 
	 *
	 * @param productId the id of the product whose sales are forecasted
	 * @param forecastStart the date when the forecast starts 
	 * @return the forecast period length (in days) to use for the prediction
	 * @throws DataLoadException 
	 */
	public int estimateHorizon(String productId, Date forecastStart) throws DataLoadException;
	
}
