package ibspan.ecap.merchant.shopdecisions.forecasting;

import ibspan.ecap.merchant.shopdecisions.dataaccess.DataQueryService;
import ibspan.ecap.merchant.shopdecisions.dataaccess.ResultProcessor;
import ibspan.ecap.shared.util.PropertiesProvider;
import jade.util.Logger;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;

//TODO: Test it
public class StationaryHorizonEstimator implements HorizonEstimator {

	private static final int DEF_MIN_HORIZON = 1;
	private static final String MIN_HORIZON_PROP_NAME = "minHorizonLength";
	private static final String MIN_PERIODS_PROP_NAME = "minPeriodsCount";
	private static final int DEF_MIN_PERIODS = 4;
	private static final String REQ_STATIONARITY_RATE_PROP_NAME = "requiredStationarityRatio";
	private static final double DEF_REQ_STATIONARITY_RATE = 0.05;
	protected Logger myLogger = Logger.getMyLogger(this.getClass().getName());
		
	public class Observation
	{
		private int periodNo;
		private double value;
				
		public int getPeriodNo() {
			return periodNo;
		}
		public void setPeriodNo(int periodNo) {
			this.periodNo = periodNo;
		}
		public double getValue() {
			return value;
		}
		public void setValue(double value) {
			this.value = value;
		}		
	}
	public int estimateHorizon(String productId, Date forecastStart) throws DataLoadException {
		
		PropertiesProvider propsProvider = PropertiesProvider.getInstance();
			
		List<Observation> data;
		try {
			data = fetchData(productId, forecastStart);
		} catch (SQLException e) {
			myLogger.log(Level.WARNING, "Error while retrieving historical data for estimating forecast horizon.", e);
			throw new DataLoadException("Error while retrieving historical data for estimating forecast horizon.",e);
		}
//		try {
//			data = fetchTestData();
//		} catch (NumberFormatException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//			throw new DataLoadException("Error while retrieving historical data for estimating forecast horizon.",e);
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//			throw new DataLoadException("Error while retrieving historical data for estimating forecast horizon.",e);
//		}

		int minPeriods = propsProvider.getIntProperty(MIN_PERIODS_PROP_NAME, DEF_MIN_PERIODS);
		int minHorizon = propsProvider.getIntProperty(MIN_HORIZON_PROP_NAME, DEF_MIN_HORIZON);
		double requiredStationarityRate = propsProvider.getDoubleProperty(REQ_STATIONARITY_RATE_PROP_NAME, DEF_REQ_STATIONARITY_RATE);
		
		int maxHorizon = Math.max(minHorizon, data.size()/minPeriods);
		
		int horizon = maxHorizon;
		boolean isStationary = false;
		
		while(!isStationary && horizon > minHorizon)
		{
			double stationaryRate = 0;
			
			List<List<Observation>> divisions = divideData(data, horizon);
			
			for (List<Observation> div : divisions) {
				double p = calculateP(div, minHorizon);
				stationaryRate += p;
				
				//logging
				if(myLogger.isLoggable(Level.FINER))
				{
					myLogger.finer("Horizon = "+horizon+"; Values: "+listContents(div)+"\np="+p);					
				}
			}
			
			isStationary = stationaryRate > requiredStationarityRate * divisions.size();
			if(!isStationary)
				horizon--;
		}
		return horizon;		
	}
	

	private String listContents(List<Observation> div) {
		StringBuilder sb = new StringBuilder();
		for (Observation observation : div) {
			sb.append(observation.value+"; ");
		}
		return sb.toString();
	}


	private List<Observation> fetchTestData() throws NumberFormatException,
			IOException {
		FileReader rd = new FileReader("data.csv");
		BufferedReader br = new BufferedReader(rd);
		String line;
		int i = 0;

		List<Observation> list = new ArrayList<Observation>();
		
		while ((line = br.readLine()) != null) {

			String regex = "\t";
			String[] params = line.split(regex);
			System.out.printf("Confirmed: %1$s\n", params[0]);
			Observation obs = new Observation();
			obs.setValue(Double.parseDouble(params[0]));
			obs.setPeriodNo(i);
			list.add(obs);
			i++;
		}
		return list;
	}
//	private boolean isStationary(List<Observation> list, int minHorizon) {
//		List<List<Observation>> divisions = divideData(list, minHorizon);
//		
//		int cutOffCount = 0;
//		for (List<Observation> div : divisions) {
//			if(isCutOff(div, minHorizon))
//			{
//				cutOffCount++;
//			}
//		}
//		
//		return cutOffCount < 0.1 * divisions.size();
//	}
	
	private double calculateP(List<Observation> sample, int seriesLength)
	{
		double[] vals = new double[sample.size()];
		for (int i = 0; i < sample.size(); i++) {
			vals[i] = sample.get(i).value;
		}
		
		double median = average(sample);//Median.find(vals, 0, vals.length-1);		//median or mean?
		
		int aboveCount = 0;
		int belowCount = 0;
		int runCount = 1;
		double prevAvg = median;
		double currentAvg = median;
		
		for (Observation obs : sample) {
			prevAvg = currentAvg;
			
			currentAvg = obs.getValue();
			
			if(currentAvg > median)
			{
				aboveCount++;
				if(prevAvg < median)
					runCount++;
			}
			else if(currentAvg < median)
			{
				belowCount++;
				if(prevAvg > median)
					runCount++;
			}
		}
		
//		List<List<Observation>> divisions = divideData(sample, seriesLength);
//		
//		for (List<Observation> div : divisions) {
//			prevAvg = currentAvg;
//			
//			currentAvg = average(div);
//			
//			if(currentAvg > median)
//			{
//				aboveCount++;
//				if(prevAvg < median)
//					runCount++;
//			}
//			else if(currentAvg < median)
//			{
//				belowCount++;
//				if(prevAvg > median)
//					runCount++;
//			}
//		}
		if(aboveCount + belowCount == 0)
			return 1;
		
		//code taken from http://home.ubalt.edu/ntsbarsh/Business-stat/otherapplets/Stationary.htm
		//compute the expected mean and variance of R
        double expectedMean = 1 + (2*aboveCount*belowCount)/(aboveCount+belowCount);           //Mean "Mu"
        double SD1 = 2*aboveCount*belowCount*(2*aboveCount*belowCount-aboveCount-belowCount);
        double SD2 = Math.pow( (aboveCount + belowCount), 2);
        double SD3 = aboveCount + belowCount - 1;
        double stdDev = SD1 / (SD2 * SD3);           //Standard deviation "Sigma"
        double variance = Math.sqrt(stdDev); 
	//calculating P value MStyle
        double z1 = (runCount - expectedMean)/variance; 
		double z2 = Math.abs(z1);
		double z = z2;
		double  t = (z > 0) ? z : (-z);
        double P1 = Math.pow((1+t*(0.049867347 + t*(0.0211410061 
                         + t*(0.0032776263 + t*(0.0000380036 + t*(0.0000488906 
                         + t*(0.000005383))))))), -16);
        double  p1 = 1 - P1 / 2;    
        double 	p = 1-((z > 0) ? p1 : 1-p1);         //this is P-value

		return p;
	}

	private double average(List<Observation> div) {
		double sum = 0;
		for (Observation observation : div) {
			sum+=observation.value;
		}
		return sum / div.size();
	}
	private List<List<Observation>> divideData(List<Observation> data,
			int horizon) {
		List<List<Observation>> divisions = new ArrayList<List<Observation>>();
		
 		for(int i=0; i<data.size(); i+=horizon)
		{
			int from = i;
			int to = Math.min(i+horizon, data.size());
			if(i+horizon<=data.size())			//trim non-complete parts
				divisions.add(data.subList(from, to));
		}
		return divisions;
	}
	private List<Observation> fetchData(String productId, Date forecastStart) throws SQLException {
		String query = "SELECT ddt_dayinepoch as periodnumber,"
		+ " COALESCE(SUM(fsa_quantityconfirmed),0) as quantityconfirmed FROM ft_sales"
		+ " RIGHT JOIN dim_product ON fsa_dpd_productid=dpd_id"
		+ " RIGHT JOIN dim_date ON fsa_ddt_negoenddateid=ddt_id"
		+ " WHERE (dpd_globalproductid = '" + productId
		+ "' OR dpd_globalproductid IS NULL)" 
		+ " AND ddt_fulldate < '"	+ DateFormat.getDateInstance().format(forecastStart) + "'"
		+ " GROUP BY (ddt_dayinepoch)"
		+ " ORDER BY (ddt_dayinepoch)";
		
		final List<Observation> data = new ArrayList<Observation>();
		
		DataQueryService.getInstance().executeQuery(query,
				new ResultProcessor() {
					public void processResult(ResultSet rs) throws SQLException {
						boolean nonZeroFound = false;
						while (rs.next()) {
							int periodNo = rs.getInt("periodnumber");
							int quantityConfirmed = rs
									.getInt("quantityconfirmed");

							if (quantityConfirmed > 0 && !nonZeroFound)
								nonZeroFound = true;
							if (nonZeroFound) {
								Observation point = new Observation();
								point.setPeriodNo(periodNo);
								point.setValue(quantityConfirmed);
								data.add(point);
							}
						}
					}
				});
		
		return data;
	}
	
	
// int startDay = DateTool.getDayInEpoch(startDate);
// int shift = startDay % periodLength;
//	
// String query = "SELECT (ddt_dayinepoch-" + shift + ")/" + periodLength+ " as
// periodnumber,"
// + " COALESCE(SUM(fsa_quantityconfirmed),0) as quantityconfirmed FROM
// ft_sales"
// + " RIGHT JOIN dim_product ON fsa_dpd_productid=dpd_id"
// + " RIGHT JOIN dim_date ON fsa_ddt_negoenddateid=ddt_id"
// + " WHERE (dpd_globalproductid = '" + productId
// + "' OR dpd_globalproductid IS NULL)"
// + " AND ddt_fulldate < '" + DateFormat.getDateInstance().format(startDate) +
// "'"
// + " GROUP BY (ddt_dayinepoch-" + shift + ")/" + periodLength
// + " ORDER BY (ddt_dayinepoch-" + shift + ")/" + periodLength;
//	
//	
// final DataSet data = new DataSet();
// data.setPeriodsPerYear(365/periodLength);
// data.setTimeVariable(timeVariableName);
//	
// DataQueryService.getInstance().executeQuery(query, new ResultProcessor()
// {
// public void processResult(ResultSet rs) throws SQLException
// {
// boolean nonZeroFound = false;
// while(rs.next())
// {
// int periodNo = rs.getInt("periodnumber");
// int quantityConfirmed = rs.getInt("quantityconfirmed");
//				
// if(quantityConfirmed > 0 && !nonZeroFound)
// nonZeroFound = true;
// if(nonZeroFound)
// {
// DataPoint point = new Observation(quantityConfirmed);
// point.setIndependentValue("period", periodNo);
// data.add(point);
// }
// }
// }
// });
//	
// return data;

}
