package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import ibspan.ecap.shared.util.TimeConverter;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

/**
 * A class responsible for gethering and managing of previous negotiation descriptions of products.
 * Implemented using a simple map, hence it doesn't provide persistence.
 *
 * @author Micha Drozdowicz
 *
 */
public class NegotiationDescriptionInventory
{
	Map<String, NegotiationDescription> lastDescriptions = new Hashtable<String, NegotiationDescription>();

	/**
	 * Returns the previous negotiation desription of the given product or null if none exists.
	 * 
	 * @param productId the id of the product which negotiation description is to be returned
	 * @return the previous negotiation desription of the given product or null if none exists.
	 */
	public NegotiationDescription getLastDescription(String productId)
	{		
		return lastDescriptions.get(productId);
	}

	/**
	 * Registers a product for management of negotiation descriptions.
	 *
	 * @param productId the id of the product to register.
	 */
	public void registerProduct(String productId)
	{
		lastDescriptions.put(productId, new NegotiationDescription(TimeConverter.getInstance().getDate(), productId, 0, 0, 0, "", ""));
	}

	/**
	 * Withdraws a product from management of negotiation descriptions.
	 *
	 * @param productId the id of the product to withdraw.
	 */
	public void withdrawProduct(String productId)
	{
		lastDescriptions.remove(productId);
		
	}

	/**
	 * Updates the previous negotiation description of the product. 
	 *
	 * @param productId
	 * @param description
	 */
	public void setLastDescription(String productId, NegotiationDescription description)
	{
		lastDescriptions.put(productId, description);		
	}
	
	/**
	 * Determines whether the specified product is registered for periodical template updates.
	 *
	 * @param productId the id of the product to check  
	 * @return <code>true</code> if the specified product is registered for periodical template updates
	 * <code>false</code> otherwise
	 */
	public boolean isRegistered(String productId)
	{
		return lastDescriptions.containsKey(productId);
	}
	
	/**
	 * Returns a list of ids of products registered for periodical template updates.
	 *
	 * @return a list of ids of products registered for periodical template updates.
	 */
	public List<String> getRegisteredProducts()
	{
		return new ArrayList<String>(lastDescriptions.keySet());
	}
		
}
