package ibspan.ecap.merchant.shopdecisions.negotiationpreparation;

import java.util.Date;
import java.util.concurrent.Callable;

/**
 * A  {@link Callable}  class representing a negotiation preparation task. The template is prepared by a  {@link TemplatePreparer}  specified in a constructor.
 * @author  Micha Drozdowicz
 */
public class PreparationTask implements Callable<NegotiationDescription>
{
	private String productId;
	private Date preparationDate;
	private NegotiationDescription previousTemplate;
	private TemplatePreparer preparer;
	
	/**
	 * Creates a new instance of <code>PreparationTask</code>.
	 *
	 * @param productId the id of the product for which a negotiation template is to be prepared
	 * @param preparationDate the date from which the template is to be used 
	 * @param previousTemplate the template used previously
	 * @param preparer the object responsible for preparing the template
	 */
	public PreparationTask(String productId, Date preparationDate, NegotiationDescription previousTemplate, TemplatePreparer preparer)
	{
		this.preparationDate = preparationDate;
		this.productId = productId;
		this.preparer = preparer;
	}

	public NegotiationDescription call() throws Exception
	{
		return preparer.prepareTemplate(productId, preparationDate, previousTemplate);		
	}

}
