/*****************************************************************
 E-commerce Agent-based Platform
 Copyright (C) 2006-2008 SRI PAS

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor,  Boston, MA  
 02110-1301, USA.
 *****************************************************************/
package ibspan.ecap.shared.ontology.negotiation;

import jade.content.Concept;

import java.util.Date;

/**
 * 
 * A class containing information about daily demand for a specific product.
 *
 * @author Micha Drozdowicz
 *
 */
public class DemandData implements Concept
{
	private String globalProductId;
	private Date when;
	private int cicQueryCount;
	private int shopsSellingProductCount;
	private int registeredClients;
	private int meanAmountPerClient;
	
	/**
	 * Gets the total number of clients that registered with the GA wanting to buy the product.
	 *
	 * @return the total number of clients that registered with the GA wanting to buy the product.
	 */
	public int getRegisteredClients()
	{
		return registeredClients;
	}
	/**
	 * Sets the total number of clients that registered with the GA wanting to buy the product.
	 *
	 * @param registeredClients total number of clients that registered with the GA wanting to buy the product.
	 */
	public void setRegisteredClients(int registeredClients)
	{
		this.registeredClients = registeredClients;
	}
	/**
	 * Gets the id of the product.
	 *
	 * @return the id of the product.
	 */
	public String getGlobalProductId()
	{
		return globalProductId;
	}
	/**
	 * Sets the id of the product.
	 *
	 * @param productId the id of the product.
	 */
	public void setGlobalProductId(String productId)
	{
		this.globalProductId = productId;
	}
	/**
	 * Gets the when on which the data was collected.
	 *
	 * @return the when
	 */
	public Date getWhen()
	{
		return when;
	}
	/**
	 * Sets the date on which the data was collected.
	 *
	 * @param date the date to set
	 */
	public void setWhen(Date date)
	{
		this.when = date;
	}
	/**
	 * Gets the amount of queries the CIC has registered during the day.
	 *
	 * @return the amount of CIC queries.
	 */
	public int getCicQueryCount()
	{
		return cicQueryCount;
	}
	/**
	 * Sets the amount of queries the CIC has registered during the day.
	 *
	 * @param cicQueryCount the amount of CIC queries.
	 */
	public void setCicQueryCount(int cicQueryCount)
	{
		this.cicQueryCount = cicQueryCount;
	}
	/**
	 * Gets the amount of shops selling the specified product during the day.
	 *
	 * @return the amount of shops selling the product
	 */
	public int getShopsSellingProductCount()
	{
		return shopsSellingProductCount;
	}
	/**
	 * Sets the amount of shops selling the specified product during the day.
	 *
	 * @param shopsSellingProductCount the amount of shops selling the product
	 */
	public void setShopsSellingProductCount(int shopsSellingProductCount)
	{
		this.shopsSellingProductCount = shopsSellingProductCount;
	}
	/**
	 * @return the meanAmountPerClient
	 */
	public int getMeanAmountPerClient() {
		return meanAmountPerClient;
	}
	/**
	 * @param meanAmountPerClient the meanAmountPerClient to set
	 */
	public void setMeanAmountPerClient(int meanAmountPerClient) {
		this.meanAmountPerClient = meanAmountPerClient;
	}

}
